﻿using System;
using System.Threading.Tasks;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using Syncfusion.SfSchedule.XForms;
using Xamarin.Forms;

namespace ScheduleDemo3
{
    public class MainPageViewModel : ViewModelBase
    {
        public FSMScheduleAppointment SelectedFSMScheduleAppointment
        {
            get { return _selectedFsmScheduleAppointment; }
            set
            {
                if (value != _selectedFsmScheduleAppointment)
                {
                    _selectedFsmScheduleAppointment = value;
                    if (value != null)
                        SelectedFSMScheduleAppointmentInfo = value.Subject;
                    else
                    {
                        SelectedFSMScheduleAppointmentInfo = String.Empty;
                    }
                    RaisePropertyChanged(() => SelectedFSMScheduleAppointment);
                }
            }
        }

        public ScheduleAppointmentCollection ScheduleAppointments
        {
            get
            {
                if (_scheduleAppointments == null)
                    _scheduleAppointments = new ScheduleAppointmentCollection();
                return _scheduleAppointments;
            }
            set
            {
                if (value != _scheduleAppointments)
                {
                    _scheduleAppointments = value;
                    RaisePropertyChanged(() => ScheduleAppointments);
                }
            }
        }

        public ScheduleView SelectedScheduleView
        {
            get { return ScheduleView.WeekView; }
        }

        public RelayCommand<Tuple<DateTime, DateTime>> SetDateRangeCommand { get; set; }

        public string SelectedFSMScheduleAppointmentInfo
        {
            get
            {
                return _selectedFSMScheduleAppointmentInfo;
            }
            set
            {
                if (value != _selectedFSMScheduleAppointmentInfo)
                {
                    _selectedFSMScheduleAppointmentInfo = value;
                    RaisePropertyChanged(() => SelectedFSMScheduleAppointmentInfo);
                }
            }
        }

        public DateTime SelectedDate
        {
            get { return _selectedDate; }
            set
            {
                if (value != _selectedDate)
                {
                    _selectedDate = value;
                    RaisePropertyChanged(() => SelectedDate);

                }
            }
        }

        public MainPageViewModel()
        {
            SetDateRangeCommand = new RelayCommand<Tuple<DateTime, DateTime>>(SetDateRange, CanSetDateRange);
        }

        #region Private methods

        private bool CanSetDateRange(Tuple<DateTime, DateTime> tuple)
        {
            return true;
        }

        private void SetDateRange(Tuple<DateTime, DateTime> tuple)
        {
            Task.Run(() =>
            {
                Task.Delay(100);
                Device.BeginInvokeOnMainThread(() =>
                {
                    ScheduleAppointments.Clear();
                    var start = tuple.Item1;
                    for (int index = 0; index < 20; index++)
                    {
                        ScheduleAppointments.Add(new FSMScheduleAppointment
                        {
                            StartTime = start + TimeSpan.FromHours(index * 2),
                            EndTime = start + TimeSpan.FromHours(index * 2 + 1),
                            Subject = index % 2 == 0
                                ? "test" + index
                                : "very long text thar makes the line grow in size thru wordwrap, and than you see the effect I descript, hopefully",
                            Color = Color.LightSkyBlue
                        });
                    }
                });
            });
        }

        #endregion

        #region Fields

        private FSMScheduleAppointment _selectedFsmScheduleAppointment;
        private ScheduleAppointmentCollection _scheduleAppointments;
        private DateTime _selectedDate;
        private string _selectedFSMScheduleAppointmentInfo;

        #endregion
    }
}