﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Input;
using Syncfusion.SfSchedule.XForms;
using Xamarin.Forms;

namespace ScheduleDemo3
{
    public class SfScheduleBehavoir : Behavior<SfSchedule>
    {
        public FSMScheduleAppointment SelectedFSMScheduleAppointment
        {
            get { return (FSMScheduleAppointment) GetValue(SelectedFSMScheduleAppointmentProperty); }
            set { SetValue(SelectedFSMScheduleAppointmentProperty, value); }
        }

        public SfSchedule AssociatedObject { get; private set; }

        public ICommand SetDateRangeCommand
        {
            get { return (ICommand) GetValue(SetDateRangeCommandProperty); }
            set { SetValue(SetDateRangeCommandProperty, value); }
        }

        public DateTime SelectedDate
        {
            get { return (DateTime) GetValue(SelectedDateProperty); }
            set { SetValue(SelectedDateProperty, value); }
        }

        public object LockObject { get; set; } = new object();

        #region Protected methods

        protected override void OnAttachedTo(SfSchedule bindable)
        {
            base.OnAttachedTo(bindable);
            AssociatedObject = bindable;
            bindable.BindingContextChanged += OnBindingContextChanged;
            bindable.VisibleDatesChangedEvent += VisibleDatesChanged;
            bindable.CellTapped += ScheduleCellTapped;
        }

        protected override void OnBindingContextChanged()
        {
            base.OnBindingContextChanged();
            BindingContext = AssociatedObject.BindingContext;
        }

        protected override void OnDetachingFrom(SfSchedule bindable)
        {
            base.OnDetachingFrom(bindable);
            bindable.BindingContextChanged -= OnBindingContextChanged;
            bindable.VisibleDatesChangedEvent -= VisibleDatesChanged;
            bindable.CellTapped -= ScheduleCellTapped;

            AssociatedObject = null;
        }

        #endregion

        #region Private methods

        private void OnBindingContextChanged(object sender, EventArgs e)
        {
            OnBindingContextChanged();
        }

        private void ScheduleCellTapped(object sender, CellTappedEventArgs args)
        {
            if (args.Datetime != DateTime.MinValue)
                SelectedDate = args.Datetime;
            if (args.Appointment is FSMScheduleAppointment)
                SelectedFSMScheduleAppointment = (FSMScheduleAppointment) args.Appointment;
            else
                SelectedFSMScheduleAppointment = null;
        }

        private void VisibleDatesChanged(object sender, VisibleDatesChangedEventArgs args)
        {
            lock (LockObject)
            {
                if (SetDateRangeCommand == null)
                    return;
                var startDate = args.visibleDates?[0] ?? DateTime.MinValue;
                var endDate = args.visibleDates?[args.visibleDates.Count - 1] ?? DateTime.MinValue;

                var parameter = new Tuple<DateTime, DateTime>(startDate, endDate);

                if (_startDate.Date != startDate.Date || _endDate.Date != endDate.Date)
                {
                    _startDate = startDate;
                    _endDate = endDate;
                    if (SetDateRangeCommand.CanExecute(parameter))
                        SetDateRangeCommand.Execute(parameter);
                    if (_oldDateRange != null)
                        if (_oldDateRange.Count != args.visibleDates.Count)
                            if (SelectedFSMScheduleAppointment != null)
                                Device.BeginInvokeOnMainThread(() => { AssociatedObject.MoveToDate = SelectedFSMScheduleAppointment.StartTime.Date; });
                    _oldDateRange = args.visibleDates.ToList();
                }
            }
        }

        #endregion

        #region Fields

        public static readonly BindableProperty SetDateRangeCommandProperty = BindableProperty.Create("SetDateRangeCommand",
            typeof(ICommand),
            typeof(SfScheduleBehavoir),
            null);

        public static readonly BindableProperty SelectedDateProperty = BindableProperty.Create("SelectedDate",
            typeof(DateTime),
            typeof(SfScheduleBehavoir),
            DateTime.MinValue,
            BindingMode.OneWayToSource);

        public static readonly BindableProperty SelectedFSMScheduleAppointmentProperty = BindableProperty.Create("SelectedFSMScheduleAppointment",
            typeof(FSMScheduleAppointment),
            typeof(SfScheduleBehavoir),
            null,
            BindingMode.OneWayToSource);

        private List<DateTime> _oldDateRange;
        private DateTime _startDate;
        private DateTime _endDate;

        #endregion
    }

    public class FSMScheduleAppointment : ScheduleAppointment
    {
        public string Info
        {
            get { return Subject + " " + StartTime + " " + EndTime; }
        }
    }
}